#!/usr/bin/env python
__author__ = 'user'
import subprocess
import logging
import os
import time
import sys
from termcolor import colored
home_dir = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))
log_location = os.path.join(home_dir,"log")
try:
    os.makedirs(log_location)
except:
    pass
logging.basicConfig(filename=os.path.join(log_location,"remote_install.log"),
                    filemode='a',
                    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                    datefmt='',
                    level=logging.INFO
                    )
logger = logging.getLogger()
logger.propagate = True
print("Log location " + log_location)
if sys.argv == 3 and str(sys.argv[3]).strip() == "-verbose":
    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    ch.setFormatter(formatter)
    logger.addHandler(ch)


import sys
logger.info("argument length : "+str(len(sys.argv)))
if len(sys.argv) < 3:
    logger.info("usage: \"/tar/file/path/FS-Agent.tar.gz\" \"/ip_list/path/ip_list.txt\" [-f]")
    print("usage: \"/tar/file/path/FS-Agent.tar.gz\" \"/ip_list/path/ip_list.txt\" [-f]")
    sys.exit(1)
tar_location = os.path.abspath(sys.argv[1]).strip()
logger.info(tar_location)
ip_list_location = os.path.abspath(sys.argv[2]).strip()
logger.info(ip_list_location)
if os.path.exists(tar_location):
    pass
else:
    logger.info("XYZFile not found : "+tar_location)
    sys.exit(1)
if os.path.exists(ip_list_location):
    pass
else:
    logger.info("ABCFile not found : "+ip_list_location)
    sys.exit(1)

tar_name = os.path.basename(tar_location)
ip_list_name = os.path.basename(ip_list_location)
logger.info(tar_name)
logger.info(ip_list_name)
force_installation = False
if len(sys.argv) == 4 and sys.argv[3] == "-f":
    logger.info("Force installation supplied. Existing agent will be uninstalled before installing new one.")
    force_installation = True


def execute_shell_command(ShellCommand):
    logger.info("Executing ShellCommand : "+ShellCommand)
    val1 = subprocess.Popen(ShellCommand, shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    std_out = val1.stdout.read()
    std_err = val1.stderr.read()
    logger.info("stdout : "+std_out)
    logger.info("stderr : "+std_err)
    return std_out.strip()


def execute_shell_script(ScriptPath):
    logger.info("Executing_shell_script : "+ScriptPath)
    process = subprocess.Popen(['/bin/sh','-c', ScriptPath], stdout=subprocess.PIPE)
    out = process.communicate()
    logger.info("stdout : "+out[0])
    logger.info("stderr : "+out[1])
    return out[0]

def getpass(prompt="Password: "):
    import termios, sys
    fd = sys.stdin.fileno()
    old = termios.tcgetattr(fd)
    new = termios.tcgetattr(fd)
    new[3] = new[3] & ~termios.ECHO          # lflags
    try:
        termios.tcsetattr(fd, termios.TCSADRAIN, new)
        passwd = raw_input(prompt)
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old)
    print "\n"
    return passwd


def ping_host(hostname):
    import subprocess
    ping_location = execute_shell_command("whereis ping")
    output = subprocess.Popen([ping_location, "-c 3", "-W 100", hostname], stdout=subprocess.PIPE).communicate()[0]
    logger.info(output)

    if ('unreachable' in output)or('Request timeout' in output):
        return False
    else:
        return True

def exec_sudo_command(client, command, pty=False):
    stdin, stdout, stderr = client.exec_command(command, get_pty=pty)
    if stdout.channel.closed is False: # If stdout is still open then sudo is asking us for a password
        logger.info("Supplying password.")
        stdin.write('%s\n' % pass_word)
        stdin.flush()
    return (stdin, stdout, stderr)

def install_on_machine(host,user,pass_word):

    logger.info("\n\n\n\n\ninstallation details on host : "+host+"\n\n\n\n")
    try:
        socket.inet_aton(host)
    except socket.error:
        print("SocketException captured")
        return "failure"
    # ip_list = None
    # with open(ip_list_location) as f:
    #     ip_list = f.readlines()
    # for ip in ip_list:
    #     pass
    if not ping_host(host):
        print("Host not pingable")
        return "failure"
    time_stamp = str(time.time())
    client = SSHClient()
    client.load_system_host_keys()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    logger.info("trying to connect to host "+host)
    try:
        client.connect(host, username=user, password=pass_word,look_for_keys=False)
    except Exception as e:
        logger.info(e)
        return "failure"
    destination_run_path = os.path.join("/Library","Freshservice","Freshservice-Discovery-Agent","run")
    stdin, stdout, stderr = client.exec_command("test -f " + destination_run_path +"; echo $?")
    logger.info("stderr: "+ str(stderr.readlines()))
    output = ""
    for line in stdout.readlines():
        output=output+line
    std_out =  output
    if std_out.strip() == "0":
        if not force_installation:
            return "already installed"
        logger.info("Agent already installed. Proceeding to uninstall.")
        old_agent_path = os.path.join("/Library","Freshservice","Freshservice-Discovery-Agent")
        python_location = os.path.join("python-2.7.13","bin","python2.7")
        agent_location = os.path.join("src", "FS-Agent.py")
        stdin, stdout, stderr = exec_sudo_command(client, "cd %s; sudo %s %s -uninstall_force" %(old_agent_path, python_location, agent_location), pty=True)
        logger.info("stderr (Uninstall): "+ str(stderr.readlines()))
        if stderr.readlines() != []:
            return "Error while uninsalling old agent"
        stdin, stdout, stderr = exec_sudo_command(client, "sudo rm -rf /Library/Freshservice")
        logger.info("stderr (Remove Agent Folder): "+ str(stderr.readlines()))
        stdin, stdout, stderr = exec_sudo_command(client, "sudo rm -rf /etc/Freshdesk/Freshservice/accesskey")
        logger.info("stderr (Remove Access Key): "+ str(stderr.readlines()))
        stdin, stdout, stderr = exec_sudo_command(client, "sudo rm /tmp/freshservice_agent_status")
        logger.info("stderr (Remove Agent Status): "+ str(stderr.readlines()))
        logger.info("Old agent successfully uninstalled.")


    destination_path = os.path.join("/tmp","FS-Agent-installer", time_stamp)
    logger.info("destination_path : " + destination_path)
    package_path = os.path.join(destination_path,"FS-Agent","FS-Agent.pkg")

    stdin, stdout, stderr = client.exec_command("mkdir -p "+destination_path)
    logger.info("stderr: "+ str(stderr.readlines()))

    sftp = client.open_sftp()
    tarfile_remote_path = os.path.join(destination_path, tar_name)
    logger.info(tar_location)
    logger.info(tarfile_remote_path)
    sftp.put(tar_location, tarfile_remote_path)
    sftp.close()


    stdin, stdout, stderr = client.exec_command("cd "+destination_path+"; tar xopf "+tar_name)
    logger.info("stderr: "+ str(stderr.readlines()))


    stdin, stdout, stderr = exec_sudo_command(client, "sudo -S installer -pkg "+package_path+" -target /")
    logger.info("stderr: "+ str(stderr.readlines()))
    logger.info("stdout: "+ str(stdout.readlines()))
    #sudo installer -pkg ./FS-Agent.pkg -target /
    client.close()
    return "success"

from paramiko import SSHClient
import paramiko

import socket

with open(ip_list_location, 'rU') as f:
    user = raw_input("Username: ")
    pass_word = getpass()
    for host in f:
        result = install_on_machine(host.strip(),user,pass_word)
        if result == "success":
            print colored("successful installation on "+host, "green")
            logger.info("successful installation on "+host)
        elif result == "failure":
            print colored("installation failed on "+host, "red")
            logger.info("installation failed on "+host)
        elif result == "already installed":
            print colored("agent already installed on "+host, "yellow")
            logger.info("agent already installed on "+host)
        else:
            print colored(result+" : "+host, "magenta")